import com.google.protobuf.gradle.generateProtoTasks
import com.google.protobuf.gradle.id
import com.google.protobuf.gradle.ofSourceSet
import com.google.protobuf.gradle.plugins
import com.google.protobuf.gradle.protobuf
import com.google.protobuf.gradle.protoc
import org.jetbrains.kotlin.gradle.tasks.KotlinCompile


plugins {
    id("org.springframework.boot")
    kotlin("jvm")
    kotlin("plugin.spring")
    id("com.google.protobuf")
    `java-library`
}

group = "com.example"
version = "0.0.1-SNAPSHOT"
java.sourceCompatibility = JavaVersion.VERSION_11

repositories {
    mavenCentral()
}

val springBootVersion: String by project
val grpcVersion: String by project
val protobufVersion: String by project
val annotationApiVersion: String by project
val grpcSpringBootStarterVersion: String by project
val opentelemetryInstrumentationVersion: String by project
val grpcKotlinVersion: String by project
val kotlinxVersion: String by project
dependencies {
    implementation(platform("org.springframework.boot:spring-boot-dependencies:$springBootVersion"))
    implementation("org.springframework.boot:spring-boot-starter")
    implementation("org.jetbrains.kotlin:kotlin-reflect")
    implementation("org.jetbrains.kotlin:kotlin-stdlib-jdk8")
    implementation("net.devh:grpc-spring-boot-starter:$grpcSpringBootStarterVersion") {
        exclude("io.grpc")
    }
    implementation("io.grpc:grpc-kotlin-stub:$grpcKotlinVersion") {
        exclude("io.grpc")
    }
    implementation(platform("io.grpc:grpc-bom:$grpcVersion"))
    implementation("io.grpc:grpc-stub")
    runtimeOnly("io.grpc:grpc-context")
    runtimeOnly("io.grpc:grpc-netty")
    runtimeOnly("io.grpc:grpc-services")
    implementation("io.grpc:grpc-protobuf")
    implementation(platform("com.google.protobuf:protobuf-bom:$protobufVersion"))
    implementation("com.google.protobuf:protobuf-java")
    implementation("com.google.protobuf:protobuf-java-util")
    implementation("javax.annotation:javax.annotation-api:$annotationApiVersion")
    implementation(platform("io.opentelemetry.instrumentation:opentelemetry-instrumentation-bom-alpha:$opentelemetryInstrumentationVersion"))
    implementation("io.opentelemetry.instrumentation:opentelemetry-instrumentation-api")
    implementation("io.opentelemetry.instrumentation:opentelemetry-grpc-1.5")
    implementation("io.opentelemetry:opentelemetry-sdk")
    implementation("io.opentelemetry:opentelemetry-extension-kotlin")
    runtimeOnly("io.opentelemetry.instrumentation:opentelemetry-logback-1.0")
    runtimeOnly("io.opentelemetry:opentelemetry-sdk-trace")
    implementation(platform( "org.jetbrains.kotlinx:kotlinx-coroutines-bom:$kotlinxVersion"))
    implementation("org.jetbrains.kotlinx:kotlinx-coroutines-core")
    testImplementation("org.springframework.boot:spring-boot-starter-test")
}

tasks.withType<KotlinCompile> {
    kotlinOptions {
        freeCompilerArgs = listOf("-Xjsr305=strict")
        jvmTarget = "11"
    }
}

tasks.withType<Test> {
    useJUnitPlatform()
    testLogging.showStandardStreams = true
}

protobuf {
    protoc {
        artifact = "com.google.protobuf:protoc:$protobufVersion"
    }
    plugins {
        id("grpc") {
            artifact = "io.grpc:protoc-gen-grpc-java:$grpcVersion"
        }
        id("grpckt") {
            artifact = "io.grpc:protoc-gen-grpc-kotlin:$grpcKotlinVersion:jdk7@jar"
        }
    }
    generateProtoTasks {
        ofSourceSet("main").forEach {
            it.plugins {
                id("grpc")
                id("grpckt")
            }
        }
    }
}

sourceSets {
    main {
        java {
            srcDirs(
                "build/generated/source/proto/main/grpc",
                "build/generated/source/proto/main/grpckt",
                "build/generated/source/proto/main/java"
            )
        }
    }
}
