package com.example

import io.grpc.ClientInterceptor
import io.grpc.ServerInterceptor
import io.opentelemetry.api.OpenTelemetry
import io.opentelemetry.instrumentation.grpc.v1_5.GrpcTracing
import io.opentelemetry.sdk.OpenTelemetrySdk
import net.devh.boot.grpc.client.interceptor.GrpcGlobalClientInterceptor
import net.devh.boot.grpc.common.util.InterceptorOrder
import net.devh.boot.grpc.server.interceptor.GrpcGlobalServerInterceptor
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.boot.autoconfigure.SpringBootApplication
import org.springframework.boot.runApplication
import org.springframework.context.annotation.Bean
import org.springframework.core.annotation.Order

@SpringBootApplication
class Application {
    private val LOG: Logger = LoggerFactory.getLogger(Application::class.java)

    @Bean
    fun openTelemetry(): OpenTelemetry {
        return OpenTelemetrySdk.builder()
            .build()
    }

    @Bean
    @GrpcGlobalServerInterceptor
    @Order(InterceptorOrder.ORDER_TRACING_METRICS + 1)
    fun tracingServerInterceptor(openTelemetry: OpenTelemetry): ServerInterceptor {
        LOG.info("gRPC server tracing is enabled")
        return GrpcTracing.create(openTelemetry).newServerInterceptor()
    }

    @Bean
    @GrpcGlobalClientInterceptor
    @Order(InterceptorOrder.ORDER_TRACING_METRICS + 1)
    fun tracingClientInterceptor(openTelemetry: OpenTelemetry): ClientInterceptor {
        LOG.info("gRPC client tracing is enabled")
        return GrpcTracing.create(openTelemetry).newClientInterceptor()
    }
}

fun main(args: Array<String>) {
    runApplication<Application>(*args)
}
