package com.example

import com.google.protobuf.MessageOrBuilder
import com.google.protobuf.TextFormat
import io.grpc.CallOptions
import io.grpc.Channel
import io.grpc.ClientCall
import io.grpc.ClientInterceptor
import io.grpc.ForwardingClientCall
import io.grpc.ForwardingClientCallListener
import io.grpc.MethodDescriptor
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.core.Ordered
import org.springframework.stereotype.Component
import io.grpc.Metadata as GRPCMetadata

@Component
class LoggingClientInterceptor : ClientInterceptor, Ordered {
    private val LOG: Logger = LoggerFactory.getLogger(LoggingClientInterceptor::class.java)
    private val PRINTER = TextFormat.printer().escapingNonAscii(false)

    override fun <ReqT, RespT> interceptCall(
        method: MethodDescriptor<ReqT, RespT>,
        callOptions: CallOptions,
        next: Channel
    ): ClientCall<ReqT, RespT> {
        return object : ForwardingClientCall.SimpleForwardingClientCall<ReqT, RespT>(
            next.newCall(method, callOptions)
        ) {
            override fun sendMessage(req: ReqT) {
                if (req is MessageOrBuilder) {
                    LOG.debug("Sending request: {}", PRINTER.printToString(req))
                }

                super.sendMessage(req)
            }

            override fun start(responseListener: Listener<RespT>, headers: GRPCMetadata) {
                val listener: Listener<RespT> =
                    object : ForwardingClientCallListener.SimpleForwardingClientCallListener<RespT>(responseListener) {
                        override fun onMessage(res: RespT) {
                            if (res is MessageOrBuilder) {
                                LOG.debug("Received response: {}", PRINTER.printToString(res))
                            }

                            super.onMessage(res)
                        }
                    }
                super.start(listener, headers)
            }
        }
    }

    override fun getOrder() = ORDER

    companion object {
        const val ORDER = 1
    }
}