package com.example

import io.grpc.examples.helloworld.GreeterGrpcKt
import io.grpc.examples.helloworld.HelloRequest
import kotlinx.coroutines.runBlocking
import net.devh.boot.grpc.client.inject.GrpcClient
import org.assertj.core.api.Assertions
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.boot.test.system.CapturedOutput
import org.springframework.boot.test.system.OutputCaptureExtension
import org.springframework.test.annotation.DirtiesContext

@SpringBootTest(
    webEnvironment = SpringBootTest.WebEnvironment.NONE
)
@DirtiesContext
@ExtendWith(OutputCaptureExtension::class)
class GrpcTraceIntegrationTest {
    @GrpcClient("helloworld", interceptors = [LoggingClientInterceptor::class])
    lateinit var stub: GreeterGrpcKt.GreeterCoroutineStub

    @Test
    fun testGrpcTracing(out: CapturedOutput) {
        val reply = runBlocking { stub.sayHello(HelloRequest.newBuilder().setName("John").build()) }
        Assertions.assertThat(reply.message).isEqualTo("Hello John")

        val tokens = out.out
            .split(System.lineSeparator())
            .filter { it.startsWith('[') }
            .map { line ->
                // https://stackoverflow.com/a/66557647/839733
                val group = "[^\\[\\]]*"
                "(?:(?<=\\[)$group)|(?:(?<=\\])$group\$)".toRegex()
                    .findAll(line)
                    .map { it.value }
                    .toList()
            }

        tokens.forEach {
            Assertions.assertThat(it)
                .withFailMessage("Line: %s doesn't match expected format", it)
                .hasSize(7)
        }

        listOf(GreeterImpl::class.java, LoggingClientInterceptor::class.java).forEach { clazz ->
            val name = clazz.simpleName
            val logs = tokens
                .filter { t -> t[5] == name }

            Assertions.assertThat(logs)
                .withFailMessage("No logs found for class: %s", name)
                .isNotEmpty
            // check trace
            logs.forEach {
                Assertions.assertThat(it[3])
                    .withFailMessage("Trace is empty for log: %s", it)
                    .isNotBlank
            }
            // check span
            logs.forEach {
                Assertions.assertThat(it[4])
                    .withFailMessage("Span is empty for log: %s", it)
                    .isNotBlank
            }
        }
    }
}
